-- ------------------------------------------------------------------------------------- --
-- 					Scroll Master - AddOn by Sapu (sapu94@gmail.com)			 		 --
--             http://wow.curse.com/downloads/wow-addons/details/slippy.aspx             --
-- ------------------------------------------------------------------------------------- --

-- The following functions are contained attached to this file:
-- Inscription:OpenCrafting() - opens the Inscription window and removes any filters on the shown crafts so that everything is visible
-- Inscription:IsScribe() - determines if the player is an enchanter

-- The following "global" (within the addon) variables are initialized in this file:
-- Inscription.vellumID - itemID for Inscription Vellum

-- ===================================================================================== --


-- load the parent file (ScrollMaster) into a local variable and register this file as a module
local TSM = select(2, ...)
local Inscription = TSM:NewModule("Inscription", "AceEvent-3.0")

local aceL = LibStub("AceLocale-3.0"):GetLocale("TradeSkillMaster_Crafting") -- loads the localization table
local debug = function(...) TSM:Debug(...) end -- for debugging

local function L(phrase)
	--TSM.lTable[phrase] = true
	return aceL[phrase]
end

-- opens the inscription window and removes any filters on the shown crafts so that everything is visible
function Inscription:OpenCrafting(nextFunc)
	CloseTradeSkill()
	local spellName
	if TSM.mode == "Inscription" then
		spellName = GetSpellInfo(45363) --Inscription = GetSpellInfo(45363)
	end
	Inscription:RegisterEvent("TRADE_SKILL_SHOW", function()
			Inscription:UnregisterEvent("TRADE_SKILL_SHOW")
			if nextFunc then nextFunc() end
		end)
	CastSpellByName(spellName) -- opens inscription
end

-- determines if the player is an enchanter (either /450 or /525)
function Inscription:IsScribe()
	local enchantIDs = {45357, 45358, 45359, 45360, 45361, 45363, 86008}
	for _, id in pairs(enchantIDs) do
		if IsSpellKnown(id) then return true end
	end
end

function Inscription:GetSlot(itemID)
	if not itemID then return end
	return Inscription.slot[itemID]
end

Inscription.slotList = {"Death Knight", "Druid", "Hunter", "Mage", "Paladin",
	"Priest", "Rogue", "Shaman", "Warlock", "Warrior"}

Inscription.slot = {
	[43533]=1, -- Death Knight
	[43552]=1, -- Death Knight
	[43553]=1, -- Death Knight
	[43554]=1, -- Death Knight
	[43671]=1, -- Death Knight
	[43672]=1, -- Death Knight
	[43673]=1, -- Death Knight
	[43825]=1, -- Death Knight
	[43826]=1, -- Death Knight
	[43827]=1, -- Death Knight
	[45799]=1, -- Death Knight
	[45800]=1, -- Death Knight
	[45804]=1, -- Death Knight
	[43551]=1, -- Death Knight
	[43550]=1, -- Death Knight
	[43549]=1, -- Death Knight
	[43534]=1, -- Death Knight
	[43535]=1, -- Death Knight
	[43536]=1, -- Death Knight
	[43537]=1, -- Death Knight
	[43539]=1, -- Death Knight
	[43541]=1, -- Death Knight
	[43542]=1, -- Death Knight
	[43543]=1, -- Death Knight
	[43544]=1, -- Death Knight
	[43546]=1, -- Death Knight
	[43547]=1, -- Death Knight
	[43548]=1, -- Death Knight
	[45806]=1, -- Death Knight
	[45604]=2, -- Druid
	[40916]=2, -- Druid
	[40914]=2, -- Druid
	[40913]=2, -- Druid
	[40912]=2, -- Druid
	[40909]=2, -- Druid
	[40906]=2, -- Druid
	[40903]=2, -- Druid
	[40900]=2, -- Druid
	[40919]=2, -- Druid
	[40922]=2, -- Druid
	[67485]=2, -- Druid
	[48720]=2, -- Druid
	[45603]=2, -- Druid
	[43332]=2, -- Druid
	[43331]=2, -- Druid
	[43316]=2, -- Druid
	[40924]=2, -- Druid
	[40923]=2, -- Druid
	[40897]=2, -- Druid
	[43674]=2, -- Druid
	[40896]=2, -- Druid
	[40902]=2, -- Druid
	[45623]=2, -- Druid
	[40915]=2, -- Druid
	[67484]=2, -- Druid
	[45601]=2, -- Druid
	[44928]=2, -- Druid
	[40921]=2, -- Druid
	[45602]=2, -- Druid
	[40901]=2, -- Druid
	[43334]=2, -- Druid
	[40908]=2, -- Druid
	[67486]=2, -- Druid
	[45622]=2, -- Druid
	[43335]=2, -- Druid
	[44922]=2, -- Druid
	[40899]=2, -- Druid
	[40920]=2, -- Druid
	[67487]=2, -- Druid
	[42904]=3, -- Hunter
	[42897]=3, -- Hunter
	[42898]=3, -- Hunter
	[42900]=3, -- Hunter
	[42901]=3, -- Hunter
	[42908]=3, -- Hunter
	[42909]=3, -- Hunter
	[42912]=3, -- Hunter
	[42914]=3, -- Hunter
	[42915]=3, -- Hunter
	[43338]=3, -- Hunter
	[43350]=3, -- Hunter
	[43356]=3, -- Hunter
	[45731]=3, -- Hunter
	[42905]=3, -- Hunter
	[45734]=3, -- Hunter
	[42910]=3, -- Hunter
	[42907]=3, -- Hunter
	[45733]=3, -- Hunter
	[45625]=3, -- Hunter
	[42913]=3, -- Hunter
	[43355]=3, -- Hunter
	[42911]=3, -- Hunter
	[42902]=3, -- Hunter
	[42917]=3, -- Hunter
	[42899]=3, -- Hunter
	[42906]=3, -- Hunter
	[42903]=3, -- Hunter
	[45732]=3, -- Hunter
	[43351]=3, -- Hunter
	[45735]=3, -- Hunter
	[45739]=4, -- Mage
	[42745]=4, -- Mage
	[42735]=4, -- Mage
	[42737]=4, -- Mage
	[42738]=4, -- Mage
	[42739]=4, -- Mage
	[42741]=4, -- Mage
	[42742]=4, -- Mage
	[42743]=4, -- Mage
	[42746]=4, -- Mage
	[42752]=4, -- Mage
	[42753]=4, -- Mage
	[43360]=4, -- Mage
	[43361]=4, -- Mage
	[45738]=4, -- Mage
	[44920]=4, -- Mage
	[42744]=4, -- Mage
	[43364]=4, -- Mage
	[42748]=4, -- Mage
	[42736]=4, -- Mage
	[45736]=4, -- Mage
	[63539]=4, -- Mage
	[42749]=4, -- Mage
	[43339]=4, -- Mage
	[44684]=4, -- Mage
	[42754]=4, -- Mage
	[45737]=4, -- Mage
	[45740]=4, -- Mage
	[50045]=4, -- Mage
	[43359]=4, -- Mage
	[42751]=4, -- Mage
	[63416]=4, -- Mage
	[44955]=4, -- Mage
	[41102]=5, -- Paladin
	[43867]=5, -- Paladin
	[41092]=5, -- Paladin
	[41095]=5, -- Paladin
	[41098]=5, -- Paladin
	[41099]=5, -- Paladin
	[41100]=5, -- Paladin
	[41101]=5, -- Paladin
	[41103]=5, -- Paladin
	[41105]=5, -- Paladin
	[41108]=5, -- Paladin
	[43365]=5, -- Paladin
	[43367]=5, -- Paladin
	[43868]=5, -- Paladin
	[45743]=5, -- Paladin
	[45746]=5, -- Paladin
	[45742]=5, -- Paladin
	[41106]=5, -- Paladin
	[43366]=5, -- Paladin
	[41107]=5, -- Paladin
	[41109]=5, -- Paladin
	[45747]=5, -- Paladin
	[43369]=5, -- Paladin
	[43340]=5, -- Paladin
	[41097]=5, -- Paladin
	[43368]=5, -- Paladin
	[43869]=5, -- Paladin
	[45745]=5, -- Paladin
	[41094]=5, -- Paladin
	[45741]=5, -- Paladin
	[45744]=5, -- Paladin
	[41104]=5, -- Paladin
	[41096]=5, -- Paladin
	[41110]=5, -- Paladin
	[66918]=5, -- Paladin
	[42404]=6, -- Priest
	[42403]=6, -- Priest
	[42397]=6, -- Priest
	[42398]=6, -- Priest
	[42400]=6, -- Priest
	[42402]=6, -- Priest
	[42406]=6, -- Priest
	[42408]=6, -- Priest
	[42410]=6, -- Priest
	[42411]=6, -- Priest
	[42415]=6, -- Priest
	[42416]=6, -- Priest
	[43342]=6, -- Priest
	[43371]=6, -- Priest
	[45756]=6, -- Priest
	[45757]=6, -- Priest
	[43373]=6, -- Priest
	[42414]=6, -- Priest
	[42407]=6, -- Priest
	[45753]=6, -- Priest
	[45755]=6, -- Priest
	[43374]=6, -- Priest
	[42401]=6, -- Priest
	[42396]=6, -- Priest
	[42405]=6, -- Priest
	[43372]=6, -- Priest
	[42399]=6, -- Priest
	[45760]=6, -- Priest
	[42409]=6, -- Priest
	[42417]=6, -- Priest
	[43370]=6, -- Priest
	[42412]=6, -- Priest
	[45758]=6, -- Priest
	[45766]=7, -- Rogue
	[42955]=7, -- Rogue
	[42956]=7, -- Rogue
	[42957]=7, -- Rogue
	[42958]=7, -- Rogue
	[42960]=7, -- Rogue
	[42961]=7, -- Rogue
	[42966]=7, -- Rogue
	[42970]=7, -- Rogue
	[42972]=7, -- Rogue
	[42973]=7, -- Rogue
	[42974]=7, -- Rogue
	[43343]=7, -- Rogue
	[43377]=7, -- Rogue
	[43379]=7, -- Rogue
	[43380]=7, -- Rogue
	[45768]=7, -- Rogue
	[43376]=7, -- Rogue
	[42967]=7, -- Rogue
	[45767]=7, -- Rogue
	[45761]=7, -- Rogue
	[45762]=7, -- Rogue
	[45764]=7, -- Rogue
	[45769]=7, -- Rogue
	[42971]=7, -- Rogue
	[42959]=7, -- Rogue
	[42954]=7, -- Rogue
	[42968]=7, -- Rogue
	[43378]=7, -- Rogue
	[42969]=7, -- Rogue
	[42963]=7, -- Rogue
	[42964]=7, -- Rogue
	[42962]=7, -- Rogue
	[64493]=7, -- Rogue
	[42965]=7, -- Rogue
	[63420]=7, -- Rogue
	[45777]=8, -- Shaman
	[41530]=8, -- Shaman
	[43388]=8, -- Shaman
	[41526]=8, -- Shaman
	[41531]=8, -- Shaman
	[41532]=8, -- Shaman
	[41533]=8, -- Shaman
	[41534]=8, -- Shaman
	[41536]=8, -- Shaman
	[41537]=8, -- Shaman
	[41540]=8, -- Shaman
	[41541]=8, -- Shaman
	[41547]=8, -- Shaman
	[43386]=8, -- Shaman
	[43725]=8, -- Shaman
	[45770]=8, -- Shaman
	[41518]=8, -- Shaman
	[41539]=8, -- Shaman
	[43385]=8, -- Shaman
	[45771]=8, -- Shaman
	[45772]=8, -- Shaman
	[41529]=8, -- Shaman
	[44923]=8, -- Shaman
	[45778]=8, -- Shaman
	[41527]=8, -- Shaman
	[41552]=8, -- Shaman
	[45776]=8, -- Shaman
	[43344]=8, -- Shaman
	[41517]=8, -- Shaman
	[41538]=8, -- Shaman
	[41524]=8, -- Shaman
	[41542]=8, -- Shaman
	[41535]=8, -- Shaman
	[43381]=8, -- Shaman
	[45775]=8, -- Shaman
	[45782]=9, -- Warlock
	[42459]=9, -- Warlock
	[42461]=9, -- Warlock
	[42462]=9, -- Warlock
	[42464]=9, -- Warlock
	[42465]=9, -- Warlock
	[42467]=9, -- Warlock
	[42470]=9, -- Warlock
	[42471]=9, -- Warlock
	[42472]=9, -- Warlock
	[42473]=9, -- Warlock
	[43389]=9, -- Warlock
	[43390]=9, -- Warlock
	[43391]=9, -- Warlock
	[45785]=9, -- Warlock
	[45789]=9, -- Warlock
	[42458]=9, -- Warlock
	[42456]=9, -- Warlock
	[45783]=9, -- Warlock
	[45779]=9, -- Warlock
	[45780]=9, -- Warlock
	[45781]=9, -- Warlock
	[43394]=9, -- Warlock
	[42453]=9, -- Warlock
	[42457]=9, -- Warlock
	[42463]=9, -- Warlock
	[42466]=9, -- Warlock
	[42460]=9, -- Warlock
	[43393]=9, -- Warlock
	[42468]=9, -- Warlock
	[43392]=9, -- Warlock
	[42454]=9, -- Warlock
	[42455]=9, -- Warlock
	[50077]=9, -- Warlock
	[63481]=10, -- Warrior
	[43395]=10, -- Warrior
	[43397]=10, -- Warrior
	[43399]=10, -- Warrior
	[43400]=10, -- Warrior
	[43412]=10, -- Warrior
	[43413]=10, -- Warrior
	[43414]=10, -- Warrior
	[43416]=10, -- Warrior
	[43417]=10, -- Warrior
	[43421]=10, -- Warrior
	[43422]=10, -- Warrior
	[43425]=10, -- Warrior
	[43427]=10, -- Warrior
	[43430]=10, -- Warrior
	[43431]=10, -- Warrior
	[43428]=10, -- Warrior
	[67483]=10, -- Warrior
	[43418]=10, -- Warrior
	[43419]=10, -- Warrior
	[45790]=10, -- Warrior
	[45792]=10, -- Warrior
	[49084]=10, -- Warrior
	[45795]=10, -- Warrior
	[43396]=10, -- Warrior
	[43398]=10, -- Warrior
	[67482]=10, -- Warrior
	[45797]=10, -- Warrior
	[43423]=10, -- Warrior
	[45794]=10, -- Warrior
	[43424]=10, -- Warrior
	[43415]=10, -- Warrior
	[43432]=10, -- Warrior
	[45793]=10, -- Warrior
}